@@ -4,7 +4,7 @@
 
     modules for backward compatibility
 
-    :copyright: Copyright 2007-2019 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2020 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
@@ -15,27 +15,14 @@ from typing import Any, Dict
 from docutils.utils import get_source_line
 
 from sphinx import addnodes
-from sphinx.config import Config
-from sphinx.deprecation import RemovedInSphinx30Warning, RemovedInSphinx40Warning
+from sphinx.deprecation import RemovedInSphinx40Warning
 from sphinx.transforms import SphinxTransform
-from sphinx.util import import_object
 
 if False:
     # For type annotation
     from sphinx.application import Sphinx
 
 
-def deprecate_source_parsers(app: "Sphinx", config: Config) -> None:
-    if config.source_parsers:
-        warnings.warn('The config variable "source_parsers" is deprecated. '
-                      'Please update your extension for the parser and remove the setting.',
-                      RemovedInSphinx30Warning)
-        for suffix, parser in config.source_parsers.items():
-            if isinstance(parser, str):
-                parser = import_object(parser, 'source parser')
-            app.add_source_parser(suffix, parser)
-
-
 def register_application_for_autosummary(app: "Sphinx") -> None:
     """Register application object to autosummary module.
 
@@ -53,7 +40,7 @@ class IndexEntriesMigrator(SphinxTransform):
     """Migrating indexentries from old style (4columns) to new style (5columns)."""
     default_priority = 700
 
-    def apply(self, **kwargs) -> None:
+    def apply(self, **kwargs: Any) -> None:
         for node in self.document.traverse(addnodes.index):
             for i, entries in enumerate(node['entries']):
                 if len(entries) == 4:
@@ -65,7 +52,6 @@ class IndexEntriesMigrator(SphinxTransform):
 
 def setup(app: "Sphinx") -> Dict[str, Any]:
     app.add_transform(IndexEntriesMigrator)
-    app.connect('config-inited', deprecate_source_parsers)
     app.connect('builder-inited', register_application_for_autosummary)
 
     return {
